; Orton Z80 computer simple echo program
; Uses interrupt mode 1 (restart at 38H)
; Serial drivers timed for 9600 baud, no parity, one stop bit
; Intended to be basis of future loader and driver for MS Basic

; Jump at reset entry point
	.ORG	0000H
	JP	L0		; NB. This instruction gets
;				;  over-written!

; Initialisation
	.ORG	0040H
L0:	LD	SP,7FFEH	; Stack pointer near RAM top
	LD	HL,3333H	; Put INC SP instructions
	LD	(38H),HL	;  at 38H, 39H
	LD	A,0E9H		; Put a JP (HL) instruction
	LD	(3AH),A		;  at 3AH
	LD	HL,47EDH	; Put a LD I,A instruction
	LD	(7FFEH),HL	;  at 7FFEH/FFFEH
	LD	A,0E9H		; Put a JP (HL) instruction
	LD	(0),A		;  at 0000H/8000H
	IM	1		; Select interrupt mode 1

; Main loop
L1:	CALL	GETCH
	CALL	PUTCH
	JR	L1

; Get character from console
; Character returned in A
; Service routine at 38H discards pushed PC and jumps to (HL)
; Uses A, B, C, HL
GETCH:	LD	HL,L2		; Wait for start bit
L2:	EI
	NOP
	DI
	LD	BC,0D09H	; Wait half a bit time
L3:	DJNZ	L3
	EI			; Check still there
	NOP
	DI
	LD	HL,L7		; Shift in 9 bits:
	LD	B,3
L4:	DJNZ	L4
L5:	LD	B,27		;  Wait whole bit time
L6:	DJNZ	L6
	SCF
	EI
	NOP
	DI
	CCF
	LD	R,A		;  (just killing time here)
	NOP
	NOP
	NOP
L7:	RRA			;  Shift in bit
	DEC	C
	JR	NZ,L5
	RLA			; Recover bit 0
	RET

; Put character to console
; Character expected in A
; A15 bit banged by jumping between RAM images
; LD I,A instruction used to control A15 during refresh
; Uses A, B, C, D, HL 
PUTCH:	LD	C,A		; Save character in C
	LD	D,10		; Send ten bits:
L8:	LD	HL,L9+8000H	;  0 bit - move to upper image
	LD	A,80H
	JP	7FFEH		;  Upper image execution:
L9:	LD	B,26		;   Wait one bit time
L10:	DJNZ	L10
	NOP
	NOP
	SCF
	RR	C
	DEC	D		;   (No bit count test needed)
	JR	C,L12		;   Next bit 0 - stay in upper image
	LD	B,2
L11:	DJNZ	L11
	NOP
	JR	L9
L12:	LD	HL,L13		;  1 bit - move to lower image
	LD	A,0
	JP	0FFFEH		;  Lower image execution:
L13:	LD	B,26		;   Wait one bit time
L14:	DJNZ	L14
	SCF
	RR	C
	DEC	D		;   Test if all bits sent
	RET 	Z
	NOP
	JR	NC,L8		;   Next bit 1 - stay in lower image
	LD	B,2
L15:	DJNZ	L15
	NOP
	JR	L13

	.END
